SystemOrganization addCategory: #ExtentionsForKendrick!

Object subclass: #KendrickEquations
	instanceVariableNames: 'eqAsSymExpr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!KendrickEquations methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 13:05'!
convertToCompatibleBackendRepresentation

	^ ((eqAsSymExpr asString
			copyReplaceAll: '#' with: '' asTokens: false)
			trimBoth: [ :c | (c = $() | (c = $)) ]) 
			findTokens: '''.'''! !

!KendrickEquations methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 17:57'!
with: symExpression

	eqAsSymExpr := symExpression ! !

Object subclass: #MeaslesScripts
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!MeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 13:05'!
after

Kendrick model
	id: 'Measles';
	population: 1000;
	status: #(S E I R); 
	S: 99999;
	I: 1;
	others: 0;
	beta: 0.0000214;
	gamma: 0.143;
	mu: 0.000351;
	sigma: 0.125;
	equations: #(
	   S:t=mu*N - beta*S*I - mu*S.
      E:t=beta*S*I - sigma*E - mu*E.
      I:t=sigma*E - gamma*I - mu*I.
      R:t=gamma*I - mu*R.
	).
	
Simulation rungeKutta
	id: 'RKSim';
	for: 'Measles';
	from: 0.0; 
	to: 150; 
	step: 1.
	
Visualization diagram 
	id: 'DiagramViz';
	for: 'RKSim';
	xLabel: 'Time (days)'.! !

!MeaslesScripts methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/5/2016 13:10'!
before

| model simulator diag |
model := KEModel new population: (KEPopulation size: 100000).
model addAttribute: #status value: #(S E I R).
model atCompartment: { #status->#S } put: 99999 atOthersPut: 0.
model atCompartment: { #status->#I } put: 1.
model addEquations: { 
            'S:t=mu*N - beta*S*I - mu*S'.
            'E:t=beta*S*I - sigma*E - mu*E'.
            'I:t=sigma*E - gamma*I - mu*I'.
            'R:t=gamma*I - mu*R'
            }.
model addParameters: {
        #beta->0.0000214. 
        #gamma->0.143. 
        #mu->0.0000351. 
        #sigma->0.125 }.
simulator := KESimulator new: #RungeKutta from: 0.0 to: 150 step: 1.
simulator executeOn: model.
diag := (KEDiagramBuilder new) data: simulator allTimeSeries.
diag xLabel: 'Time (days)'.
diag open! !

Error subclass: #KendrickDSLSyntaxError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

ProtoObject subclass: #KendrickInternalDSLKeywords
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

KendrickInternalDSLKeywords subclass: #Kendrick
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Kendrick class methodsFor: 'public-api' stamp: 'NickPapoulias 4/4/2016 17:44'!
model

	^ KendrickModel new! !

Kendrick subclass: #KendrickModel
	instanceVariableNames: 'population attributes parameters equations keModel explicitelySetCompartments otherCompartmentValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 4/4/2016 17:33'!
atCompartment: variable value: value

	|assoc|

	assoc := (attributes associations select: [ :as | as value includes: variable ]) first.
	keModel at: { assoc key -> assoc value } put: value.! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 4/4/2016 17:28'!
attributeValuesIncludes: variable

	^ (attributes values select: [ :values | values includes: variable ]) isEmpty not! !

!KendrickModel methodsFor: 'public-api' stamp: 'NickPapoulias 4/4/2016 17:40'!
doesNotUnderstand: aMessage

	|variable value|
	
	variable := aMessage selector.
	value := aMessage args first.

	value isNumber 
		ifTrue: [   
			(self attributeValuesIncludes: variable)
				ifTrue: [ 
					explicitelySetCompartments add: variable.
					self atCompartment: variable value: value. 
				]
				ifFalse: [ 
					parameters at: variable put: value.
					keModel addParameter: variable value: value 
				]
		] ifFalse: [  
			value isArray
				ifTrue: [ 
					attributes at: variable put: value.
					keModel addAttribute: variable value: value  
				]
				ifFalse: [ KendrickDSLSyntaxError signal: 'Expected a Number or Array got: ' , value asString , ' instead'. ]	
		]
	
	! !

!KendrickModel methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 13:06'!
equations: anArrayOfSymExpressions

	equations :=
		KendrickEquations new with: anArrayOfSymExpressions.
	keModel addEquations: equations convertToCompatibleBackendRepresentation.
	! !

!KendrickModel methodsFor: 'protected' stamp: 'NickPapoulias 4/4/2016 17:41'!
initialize

	keModel := KEModel new.
	attributes := Dictionary new.
	parameters := Dictionary new.
	explicitelySetCompartments := OrderedCollection new.! !

!KendrickModel methodsFor: 'public-api' stamp: 'NickPapoulias 4/4/2016 17:42'!
others: anInteger

	otherCompartmentValue := anInteger.

	keModel initCompartments
		keysDo: [ :k | 
			(explicitelySetCompartments includes: k)
				ifFalse: [ self atCompartment: k put: anInteger ] 
		]! !

!KendrickModel methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 13:11'!
population: anInteger

	keModel population: (KEPopulation size: anInteger).! !

!KendrickInternalDSLKeywords class methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 16:53'!
withId: anId

	^ self allInstances select: [ :instance | instance id = anId ]! !

!KendrickInternalDSLKeywords methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 16:51'!
id

	^ id! !

!KendrickInternalDSLKeywords methodsFor: 'as yet unclassified' stamp: 'NickPapoulias 4/4/2016 16:52'!
id: aString

	id := aString! !

KendrickInternalDSLKeywords subclass: #Simulation
	instanceVariableNames: 'algo model from to step simulator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Simulation class methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:24'!
rungeKutta

	^ self new 
		with: #RungeKutta! !

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 4/5/2016 15:38'!
allTimeSeries

	^ simulator allTimeSeries ! !

!Simulation methodsFor: 'protected' stamp: 'NickPapoulias 4/5/2016 15:33'!
execute

	simulator := KESimulator new: algo from: from to: to step: step.
	simulator executeOn: model.! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:26'!
for: aModelID

	model := KendrickModel withId: aModelID.! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:27'!
from: aNumber

	from := aNumber! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:29'!
step: aNumber

	step := aNumber! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:27'!
to: aNumber

	to := aNumber! !

!Simulation methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:24'!
with: anAlgorithm

	algo := anAlgorithm ! !

KendrickInternalDSLKeywords subclass: #Visualization
	instanceVariableNames: 'simulation xLabel diag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ExtentionsForKendrick'!

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:36'!
for: simulationID

	simulation := Simulation withID: simulationID.! !

!Visualization methodsFor: 'protected' stamp: 'NickPapoulias 4/5/2016 15:37'!
open

	diag := (KEDiagramBuilder new) data: simulation allTimeSeries.
	diag xLabel: xLabel.
	diag open! !

!Visualization methodsFor: 'public-api' stamp: 'NickPapoulias 4/5/2016 15:36'!
xLabel: aString

	xLabel := aString! !
